<?php
/*
** Zabbix
** Copyright (C) 2001-2021 Zabbix SIA
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
**/


function add_audit($action, $resourcetype, $note) {
	if (mb_strlen($note) > 128) {
		$note = mb_substr($note, 0, 125).'...';
	}

	$values = [
		'userid' => CWebUser::$data['userid'],
		'clock' => time(),
		'ip' => substr(CWebUser::getIp(), 0, 39),
		'action' => $action,
		'resourcetype' => $resourcetype,
		'note' => $note
	];

	try {
		DB::insert('auditlog', [$values]);

		send_auditlog_to_syslog($values);

		return true;
	}
	catch (DBException $e) {
		return false;
	}
}

function add_audit_ext($action, $resourcetype, $resourceid, $resourcename, $table_name, $values_old, $values_new) {
	$values_diff = [];

	if ($action == AUDIT_ACTION_UPDATE && !empty($values_new)) {
		foreach ($values_new as $id => $value_new) {
			// log only the values that have changed, skip arrays
			if (isset($values_old[$id])
					&& !is_array($values_old[$id])
					&& !is_array($value_new)
					&& strcmp($values_old[$id], $value_new) != 0) {
				array_push($values_diff, $id);
			}
		}
		if (count($values_diff) == 0) {
			return true;
		}
	}

	if (mb_strlen($resourcename) > 255) {
		$resourcename = mb_substr($resourcename, 0, 252).'...';
	}

	/*
	 * CWebUser is not initialized in CUser->login() method.
	 * $userid with value NULL throws DBEXECUTE_ERROR later, so no audit record will be created.
	 */
	$userid = ($action == AUDIT_ACTION_LOGIN)
		? $resourceid
		: (CWebUser::$data ? CWebUser::$data['userid'] : null);

	$values = [
		'userid' => $userid,
		'clock' => time(),
		'ip' => substr(CWebUser::getIp(), 0, 39),
		'action' => $action,
		'resourcetype' => $resourcetype,
		'resourceid' => $resourceid,
		'resourcename' => $resourcename
	];

	try {
		$auditId = DB::insert('auditlog', [$values]);
		$auditId = reset($auditId);

		if ($action == AUDIT_ACTION_UPDATE) {
			$value_details = [];
			foreach ($values_diff as $id) {
				$value_details[] = [
					'auditid' => $auditId,
					'table_name' => $table_name,
					'field_name' => $id,
					'oldvalue' => $values_old[$id],
					'newvalue' => $values_new[$id]
				];
			}
			DB::insert('auditlog_details', $value_details);
		}

		send_auditlog_to_syslog($values, isset($value_details) ? $value_details : null);

		return true;
	}
	catch (DBException $e) {
		return false;
	}
}

function add_audit_details($action, $resourcetype, $resourceid, $resourcename, $note = null, $userId = null) {
	if ($userId === null) {
		$userId = CWebUser::$data['userid'];
	}

	if (mb_strlen($resourcename) > 255) {
		$resourcename = mb_substr($resourcename, 0, 252).'...';
	}

	$values = [
		'userid' => $userId,
		'clock' => time(),
		'ip' => substr(CWebUser::getIp(), 0, 39),
		'action' => $action,
		'resourcetype' => $resourcetype,
		'resourceid' => $resourceid,
		'resourcename' => $resourcename,
		'note' => $note
	];
	try {
		DB::insert('auditlog', [$values]);

		send_auditlog_to_syslog($values);

		return true;
	}
	catch (DBException $e) {
		return false;
	}
}

function send_auditlog_to_syslog($values, $value_details = null) {
	global $AUDITLOG_SYSLOG;
	if (!isset($AUDITLOG_SYSLOG)) {
		return;
	}

	// API::User()->get() does not work here in login failure event because of no login session
	// and no permission to access to API. So we get data directly from DB.
	$u = DB::select('users', ['output' => ['alias'], 'userids' => [$values['userid']]]);
	$user = isset($u[0]) ? $u[0]['alias'] : $values['userid'];

	$r = CControllerAuditLogList::getResourcesList();
	$resource = isset($r[$values['resourcetype']]) ? $r[$values['resourcetype']] : 'Unknown resource';

	$a = CControllerAuditLogList::getActionsList();
	$action = isset($a[$values['action']]) ? $a[$values['action']] : 'Unknown action';

	$resourceid = isset($values['resourceid']) ? $values['resourceid'] : 0;
	$desc = isset($values['resourcename']) ? $values['resourcename'] : '';
	$details = isset($value_details)
		? implode(', ', array_map('auditlog_details_str', $value_details))
		: (isset($values['note']) ? $values['note'] : '');

	$message = "Time={$values['clock']} User={$user} " .
		"IP={$values['ip']} Resource={$resource} " .
		"Action={$action} ID={$resourceid} " .
		"Description={$desc} Details={$details}";

	$facility = isset($AUDITLOG_SYSLOG['FACILITY']) ? $AUDITLOG_SYSLOG['FACILITY'] : LOG_USER;
	$priority = isset($AUDITLOG_SYSLOG['PRIORITY']) ? $AUDITLOG_SYSLOG['PRIORITY'] : LOG_INFO;

	openlog('zabbix_web', LOG_PID, $facility);
	syslog($priority, $message);
	closelog();
}

function auditlog_details_str($v) {
	$old = isset($v['old_value']) ? $v['old_value'] : (isset($v['oldvalue']) ? $v['oldvalue'] : '');
	$new = isset($v['new_value']) ? $v['new_value'] : (isset($v['newvalue']) ? $v['newvalue'] : '');
	return $v['table_name'].'.'.$v['field_name'].': '.$old.' => '.$new;
}
